var express = require('express');
var http = require("request");
var ipfilter = require('express-ipfilter').IpFilter;
var server_interface = express();
var fs = require("fs");
var server=null;
const path = require('path');
const Api= require(path.join(__dirname, 'Api.js'));
const NetworkDiscovery=require(path.join(__dirname, 'NetworkDiscovery.js'));
const EventLog= require(path.join(__dirname, 'EventLog.js'));
const { exec } = require('child_process');
var app=null;
var ips = ['127.0.0.1'];
var lisener_added=false;
var port=7245;
let mainWindow;
var dev_mode=false;

module.exports=
{
    setMainWindow:async function(win,dir,params)
    {
        await NetworkDiscovery.setMainWindow(win,dir);
        NetworkDiscovery.checkNewAdminAppUpdate(params);
        NetworkDiscovery.StartDiscovery();
        mainWindow=win;
    },
    setAppVariable:function(value)
    {
        app=value;
    },
    killThreads:function()
    {
        NetworkDiscovery.killThreads();
    },
    setDevMode:function(value)
    {
        dev_mode=value;
    }
}
try
{
    server = server_interface.listen(port, function () {});
    lisener_added=true;
}
catch(e)
{
    checkPort();
}

server_interface.get('/isAlive', function (req, responze) 
 {   
    responze.end(JSON.stringify({alive:true}));
 });
 if(lisener_added)
 {
    checkPort();
 }
 
 function checkPort()
 {
    http.get("http://127.0.0.1:"+port+"/isAlive",{ timeout: 500 }, (error, resp, body) => 
    {
        if (error || resp.statusCode == 500 || resp.statusCode == 404) 
        {
            EventLog.writeLog(port+" is used, try to find free port");
            server_interface.removeAllListeners();
            port++;
            lisener_added=false;
            try
            {
                server = server_interface.listen(port, function () {});
                lisener_added=true;
            }
            catch(e)
            {
                checkPort();
            }
            if(lisener_added==true)
            {
                checkPort();
            }
        }
        else if (resp && body) 
        {
            mainWindow.webContents.on('did-finish-load', () => {
                mainWindow.webContents.send("port",{port:port});
                EventLog.writeLog("Currently used port: "+port);
            });      
        }
       
    });
 }
server_interface.get('/getUser', function (req, responze) 
 {   
     try
     {
        Api.getUser(req,responze);
        
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
 server_interface.get('/lockApp', function (req, responze) 
 {   
     try
     {
        var port=req.query.port;
        var ip = req.headers['x-forwarded-for'] ||req.connection.remoteAddress ||req.socket.remoteAddress ||req.connection.socket.remoteAddress;
        ip = ip.split(',')[0];
        ip = ip.split(':').slice(-1);
        Api.lockApp(ip[0],responze,port);
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
 server_interface.get('/unlockApp', function (req, responze) 
 {   
     try
     {
         Api.unlockApp(responze);
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
server_interface.use(ipfilter(ips, {mode: 'allow',log:false}));
server_interface.get('/downloadAdminLog', function (req, responze) 
{   
    try
    {
        EventLog.downloadAdminLog(responze);
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
});
server_interface.get('/getDataService', function (req, responze) 
{   
    try
    {
        Api.getDataService(responze);
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
});
server_interface.post('/writeLog', function (req, responze) 
{   
    try
    {
        var param = '';
        responze.setHeader('Content-Type', 'application/json');
        req.on("data",function(chunk)
        {
            try
            {
                param += chunk.toString();
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });
        req.on("end",function()
        {   
            try
            {
                param=JSON.parse(param);
                EventLog.writeLog(JSON.stringify(param.msg),param.type);
                responze.end(JSON.stringify({success:true}));
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });   
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
   
 });
server_interface.post('/postDataService', function (req, responze) 
{   
    try
    {
        var param = '';
        responze.setHeader('Content-Type', 'application/json');
        req.on("data",function(chunk)
        {
            try
            {
                param += chunk.toString();
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });
        req.on("end",function()
        {   
            try
            {
                param=JSON.parse(param);
                Api.postDataService(responze,param);
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });   
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
   
 });
 server_interface.get('/clearAdminDb', function (req, responze) 
 {   
     try
     {
         var device_insert_enabled=null;
        if(req.query["device_insert_enabled"])
        {
            device_insert_enabled=(req.query["device_insert_enabled"] === 'true');
        }
         Api.clearAdminDb(responze,device_insert_enabled);
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
 server_interface.post('/updateApp', function (req, responze) 
{   
    try
    {
        var param = '';
        responze.setHeader('Content-Type', 'application/json');
        req.on("data",function(chunk)
        {
            try
            {
                param += chunk.toString();
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });
        req.on("end",function()
        {   
            try
            {
                param=JSON.parse(param);
                var commmand="";
                if(param.platform=="win32")
                {
                    commmand=param.file_dest;
                }
                else if(param.platform=="darwin")
                {
                    commmand="sudo hdiutil attach "+param.file_dest;
                }
                else
                {
                    commmand="sudo dpkg -i "+param.file_dest;
                }
                exec(commmand);
                setTimeout(function()
                {
                    app.quit();
                    responze.end(JSON.stringify({success:true}));
                },2000);
               
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });   
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
   
 });
 server_interface.get('/restartApp', function (req, responze) 
 {   
     try
     {
        app.relaunch();
        app.exit(0);
        responze.end(JSON.stringify({msg:true}));
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
 server_interface.get('/distributeDb', function (req, responze) 
 {   
     try
     {
         Api.distributeDb(responze);
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
 server_interface.post('/postOutlookCode',function(req,responze)
 {  
    var param = '';
    responze.setHeader('Content-Type', 'application/json');
    req.on("data",function(chunk){param += chunk.toString();});
    req.on("end",function()
    {   
        var params = param.slice(0).split('&');
        var paramarray = {};
        params.forEach(function(param) {
            param = param.split('=');
            paramarray[param[0]] = param[1];
        });
        var code = paramarray.code;
        responze.send("Copy the code and paste it into your application\n\n" + code);
    });
 });
 
 server_interface.get('/startLockCheck', function (req, responze) 
 {   
     try
     {
        if(dev_mode==false)
        {
            NetworkDiscovery.StartLockCheck(req.socket.localPort);
            responze.end(JSON.stringify({msg:true}));
        }
        else
        {
            responze.end(JSON.stringify({msg:false}));
        }
       
       
     }
     catch(ex)
     {
         responze.end(JSON.stringify(ex));
     }
 });
 server_interface.post('/configMsCalendar', function (req, responze) 
 {   
    try
    {
        var param = '';
        responze.setHeader('Content-Type', 'application/json');
        req.on("data",function(chunk)
        {
            try
            {
                param += chunk.toString();
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });
        req.on("end",function()
        {   
            try
            {
                param=JSON.parse(param);
                Api.configMsCalendar(responze,param);
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });   
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
 });
 server_interface.post('/manualDeviceAdd', function (req, responze) 
 {   
    try
    {
        var param = '';
        responze.setHeader('Content-Type', 'application/json');
        req.on("data",function(chunk)
        {
            try
            {
                param += chunk.toString();
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });
        req.on("end",function()
        {   
            try
            {
                param=JSON.parse(param);
                NetworkDiscovery.manualDeviceAdd(responze,param)
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });   
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
 });
 server_interface.post('/reciveDeviceDb', function (req, responze) 
 {   
    try
    {
        var param = '';
        responze.setHeader('Content-Type', 'application/json');
        req.on("data",function(chunk)
        {
            try
            {
                param += chunk.toString();
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });
        req.on("end",function()
        {   
            try
            {
                param=JSON.parse(param);
                Api.reciveDeviceDb(responze,param)
                responze.end(JSON.stringify({msg:true}));
               // Api.configMsCalendar(responze,param);
            }
            catch(ex)
            {
                responze.end(JSON.stringify(ex));
            }
           
        });   
    }
    catch(ex)
    {
        responze.end(JSON.stringify(ex));
    }
 });
 