
const spawn = require('threads').spawn;
const path = require('path');
const EventLog= require(path.join(__dirname, 'EventLog.js'));
const Database= require(path.join(__dirname, 'Database.js'));
const EncrypterDecrypter= require(path.join(__dirname, 'EncrypterDecrypter.js'));
const http = require("request");
var discovery_enabled=false;

const discovery_thread = spawn(function(params, done) 
{
    var check_subnet=true;
    var http = require("request");
    var os = require('os');
    var ifaces = os.networkInterfaces();
    var ip = require('ip');
    var CLIENT_PORT=6024;
    var dgram = require('dgram');
    var client = dgram.createSocket('udp4');
    var scan_result={subnets:[],possible_devices:[],devices:[]};
    var scan_option={possible_device_goal_count:0,possible_device_current_count:0};    
    startDiscover();
    function startDiscover()
    {
        getNetworks();
        initBroadCast();
    }
    function getNetworks()
    {
        Object.keys(ifaces).forEach(function (ifname) {
            var alias = 0;
          
            ifaces[ifname].forEach(function (iface) {
              if ('IPv4' !== iface.family || iface.internal !== false) {
                return;
              }
          
              var subnet_mask_length=ip.subnet(iface.address,iface.netmask).subnetMaskLength
              if (alias >= 1) 
              {
                scan_result.subnets.push(iface.address+"/"+subnet_mask_length);
              } else 
              {
                scan_result.subnets.push(iface.address+"/"+subnet_mask_length);
              }
              ++alias;
            });
          });
    }
    function restartDiscover()
    {
        scan_result.devices=[];
        scan_option={possible_device_goal_count:0,possible_device_current_count:0};
        setGetDeviceInfoTimer();

    }
    function initBroadCast()
    {
       
        client.on('listening', function () 
        {
            var address = client.address();
           // client.setBroadcast(true);
        });
        
        client.on('message',messageAction(scan_result));
        
        client.bind(CLIENT_PORT);
        setGetDeviceInfoTimer();
       
    }
    function setGetDeviceInfoTimer()
    {
        setTimeout(function()
        {
           getDevicesInfo();
        },15000);
    }
    function messageAction(scan_result)
    {
        return function (message, rinfo) 
        {
            
            if(message.indexOf("device_sync")!=-1 && checkAddressIsOnSubnet(rinfo.address))
            {
                var address="http://"+rinfo.address+":8080/get_db?overview_sync_enabled=true";
                var add=true;
                for(var i=0;i<scan_result.possible_devices.length;i++)
                {
                    if(scan_result.possible_devices[i]==address)
                    {
                        add=false;
                        break;
                    }
                }
                if(add==true)
                {
                    scan_result.possible_devices.push(address);
                }
            }
        }
    }
    function checkAddressIsOnSubnet(address)
    {
        if(check_subnet==true)
        {
            for(var i=0;i<scan_result.subnets.length;i++)
            {
                if(ip.cidrSubnet(scan_result.subnets[i]).contains(address))
                {
                    return true;
                }
            }
            return false;
        }
        return true;
      
    }
    function getDevicesInfo()
    {
        if(scan_result.possible_devices.length==0)
        {
            done({devices:null});
            restartDiscover();
        }
        else
        {
            scan_option.possible_device_goal_count=scan_result.possible_devices.length;
            for(var device=0;device<scan_result.possible_devices.length;device++)
            {
                http.get(scan_result.possible_devices[device],{ timeout: 10000 }, (error, response, body) => 
                {
                    if (error || response.statusCode == 500 || response.statusCode == 404 || response.statusCode == 400) 
                    {
                        checkGetDeviceInfoIsEnd();
                    }
                    else if (response && body) 
                    {
                        scan_result.devices.push(body);
                        checkGetDeviceInfoIsEnd();
                    }
                });
            }
        }
    
        
    }
    function checkGetDeviceInfoIsEnd()
    {
        scan_option.possible_device_current_count++;
        if(scan_option.possible_device_goal_count==scan_option.possible_device_current_count)
        {
            finishAndRestartDisover();
        }
    }
    function finishAndRestartDisover()
    {
        done({devices:scan_result.devices});
        restartDiscover();
    }
   
   // done({kill:true});
   
});
const check_new_admin_app_update_thread=spawn(function(params, done)
{
    var http = require("request");
    var fs = require('fs');
    const path = require('path');
    var download = require('download-file');
    
    setInterval(function()
    {
       start();
    },60000);
    start();
    function start()
    {
        var update_endpoint="";
        var uploaded_files=null;
        var ext=null;
        if(params.platform=="win32")
        {
            update_endpoint="https://tigermeeting.app/en/file-list-exe";
            ext=".exe";
        }   
        else if(params.platform=="darwin")
        {
            update_endpoint="https://tigermeeting.app/en/file-list-dmg";
            ext=".dmg";
        }   
        else if(params.platform=="linux")
        {
            update_endpoint="https://tigermeeting.app/en/file-list-deb";
            ext=".deb";
        }
        http.get(update_endpoint,{ timeout: 5000 }, (error, response, body) => 
        {
            if (error || response.statusCode == 500 || response.statusCode == 404) 
            {
                done({error:error});
            }
            else if (response && body) 
            {
                uploaded_files=JSON.parse(body);
                var biggest_version=0;
                var biggest_version_index=0
                for(var i=0;i<uploaded_files.length;i++)
                {
                    uploaded_files[i].version=uploaded_files[i].fileName.replace("Tiger-admin-","");
                    uploaded_files[i].version=uploaded_files[i].version.split('.').join("");
                    uploaded_files[i].version= uploaded_files[i].version.replace(/\D/g,'');
                    uploaded_files[i].platform=params.platform;
                    if(biggest_version<uploaded_files[i].version)
                    {
                        biggest_version=uploaded_files[i].version;
                        biggest_version_index=i;
                    }
                }
                checkAppVersions(uploaded_files[biggest_version_index]);
            }
        });       
    }
    function checkAppVersions(new_app)
    {
        var version=params.version.split('.').join("");       
        if(new_app.version>version)
        {
            done(new_app)
        } else {
            done({noNewUpdates:true});
        }
    }
});

const lock_thread=spawn(function(params, done) 
{
    var SRC_PORT = 6030;
    var SERVER_PORT = 6031;
    var CLIENT_PORT=6031;
    var MULTICAST_ADDR = [];
    var dgram = require('dgram');
    var server = dgram.createSocket("udp4");
    var client = dgram.createSocket('udp4');
    var os = require('os');
    var http = require("request");
    var ifaces = os.networkInterfaces();
    var ip = require('ip');
    var local_ips=[];
    getLocalIps();
    getMulticastAddress();
    function getMulticastAddress()
    {
        Object.keys(ifaces).forEach(function (ifname) {
            var alias = 0;
          
            ifaces[ifname].forEach(function (iface) {
              if ('IPv4' !== iface.family || iface.internal !== false) {
                return;
              }
          
              if (alias >= 1) {
                MULTICAST_ADDR.push( ip.subnet(iface.address,iface.netmask).broadcastAddress);
              } else {
                MULTICAST_ADDR.push( ip.subnet(iface.address,iface.netmask).broadcastAddress);
              }
              ++alias;
            });
          });
    }
    function getLocalIps()
    {
        Object.keys(ifaces).forEach(function (ifname) {
            var alias = 0;
          
            ifaces[ifname].forEach(function (iface) {
              if ('IPv4' !== iface.family || iface.internal !== false) {
                return;
              }
          
              if (alias >= 1) {
                local_ips.push(iface.address);
               
              } else {
                local_ips.push(iface.address);
              }
              ++alias;
            });
          });
    }
    function checkAddressIsMain(address)
    {
        for(var i=0;i<local_ips.length;i++)
        {
            if(address==local_ips[i])
            {
                return true;
            }
        }
        return false;
    }
    function sendMessage()
    {
        var msg='{"action":"lock","port":'+params.port+'}';
        var message = new Buffer(msg);
        for(var i=0;i<MULTICAST_ADDR.length;i++)
        server.send(message, 0, message.length, SERVER_PORT, MULTICAST_ADDR[i], function () {});
    }
    server.bind(SRC_PORT, function () 
    {
       server.setBroadcast(true);
       sendMessage();
       setInterval(sendMessage,5000);
      
    });
    client.on('listening', function () 
    {
        var address = client.address();
        client.setBroadcast(true);
        
    });
    client.on('message',function(message, rinfo)
    {
        message=message.toString('utf8');
        var port=7245;
        try
        {
            port=JSON.parse(message).port;
        }
        catch(ex)
        {
            console.log(ex);
        }
        if(checkAddressIsMain(rinfo.address)==false)
        { 
            http.get("http://"+rinfo.address+":7245/lockApp?port="+port+"",{ timeout: 5000 }, (error, response, body) => {});
        }
       
    });
    client.bind(CLIENT_PORT);
});

module.exports =
{
    setMainWindow:function(win,dir)
    {
        Database.setMainWindow(win,dir);
    },
    StartDiscovery: async function () 
    {
       discoverNetwork();
    },
    StartStatusUpdate:async function()
    {
        statusUpdate();
    },
    StartLockCheck:async function(port)
    {
       lockCheck(port);
    },
    manualDeviceAdd:function(responze,params)
    {
        manualAddDevice(responze,params.ip);
    },
    killThreads:function()
    {
        lock_thread.kill();
        discovery_thread.kill();
        check_new_admin_app_update_thread.kill();
    },
    checkNewAdminAppUpdate:function(params)
    {
        checkNewAdminAppUpdate(params);
    }
}
function manualAddDevice(responze,ip)
{
    http.get("http://"+ip+":8080/get_db?overview_sync_enabled=true",{ timeout: 5000 }, (error, response, body) => 
    {
        if (error || response.statusCode == 500 || response.statusCode == 404 || response.statusCode == 400) 
        {
            responze.end(JSON.stringify({error:error}));
        }
        else if (response && body) 
        {
            var result=[];

            body=JSON.parse(body);
            body=EncrypterDecrypter.decrypt(body.data);
            body = JSON.parse(body);
            result.push(body);
            Database.insertDevicesToDb(JSON.stringify(result));
            responze.end(JSON.stringify({statusCode:response.statusCode}));
        }
    });
}
function checkNewAdminAppUpdate(params)
{
    check_new_admin_app_update_thread.send(params) 
    .on('message', function(response) 
    {
        if(response.fileName || response.noNewUpdates)
        {
            Database.sendNewAdminAppUpdateNotification(response);
        }    
    })
    .on('error', function(error) 
    {
        EventLog.writeLog('Worker errored:'+JSON.stringify(error),"Error");
    })
    .on('exit', function() 
    {
        EventLog.writeLog('Worker has been terminated 2.',"Error");
    });
}
function lockCheck(port)
{
    lock_thread.send({port:port})
    .on('message', function(response) 
    {
    })
    .on('error', function(error) 
    {
        EventLog.writeLog('Worker errored:'+JSON.stringify(error),"Error");
    })
    .on('exit', function() 
    {
        EventLog.writeLog('Worker has been terminated 1.',"Error");
    });
}

async function discoverNetwork()
{
    discovery_thread.send({dir:__dirname})
    .on('message', async function(response) 
    {   
        if(response.devices==null)
        {
            Database.insertDevicesToDb(JSON.stringify(response.devices));
        }
        else
        {
            for(var i=0;i<response.devices.length;i++)
            {
                response.devices[i]=JSON.parse(response.devices[i]);
                response.devices[i]=EncrypterDecrypter.decrypt(response.devices[i].data);
                response.devices[i] = JSON.parse(response.devices[i]);
            }
            if(await Database.checkDbLocked()==false && Database.getSyncActive()==false)
            {
                Database.insertDevicesToDb(JSON.stringify(response.devices));
                if(response.kill)
                {
                    discovery_thread.kill();
                }
            }
        }
       
        if(response.kill)
        {
            discovery_thread.kill();
        }
        
    })
    .on('error', function(error) 
    {
        EventLog.writeLog('Worker errored:'+JSON.stringify(error),"Error");
    })
    .on('exit', function() 
    {
        EventLog.writeLog('Worker has been terminated 3.',"Error");
    });
}
