const electron = require('electron')
const app = electron.app
const BrowserWindow = electron.BrowserWindow
const path = require('path')
const url = require('url')
const ApiInterface= require(path.join(__dirname, 'backend/ApiInterface.js'));

let mainWindow

const isSecondInstance = app.makeSingleInstance((commandLine, workingDirectory) => {
    // Someone tried to run a second instance, we should focus our window.
    if (mainWindow) {
        if (mainWindow.isMinimized()) mainWindow.restore()
        mainWindow.focus()
    }
})

if (isSecondInstance) {
    app.quit()
}

function createWindow () {
  mainWindow = new BrowserWindow({minWidth: 1200, minHeight: 768,icon:path.join(__dirname, '/icons/tiger_x3S_1.ico')})
  mainWindow.loadURL(url.format({
    pathname: path.join(__dirname, '/frontend/index.html'),
    protocol: 'file:',
    slashes: true
  }))
  if (process.platform === 'darwin') { 
    var template = [{ 
      label: 'FromScratch', 
      submenu: [{ 
        label: 'Quit', 
        accelerator: 'CmdOrCtrl+Q', 
        click: function() { 
          app.quit(); 
        } 
      }] 
    }, { 
      label: 'Edit', 
      submenu: [{ 
        label: 'Undo', 
        accelerator: 'CmdOrCtrl+Z', 
        selector: 'undo:' 
      }, { 
        label: 'Redo', 
        accelerator: 'Shift+CmdOrCtrl+Z', 
        selector: 'redo:' 
      }, { 
        type: 'separator'
      }, { 
        label: 'Cut', 
        accelerator: 'CmdOrCtrl+X', 
        selector: 'cut:' 
      }, { 
        label: 'Copy', 
        accelerator: 'CmdOrCtrl+C', 
        selector: 'copy:'
      }, { 
        label: 'Paste', 
        accelerator: 'CmdOrCtrl+V', 
        selector: 'paste:' 
      }, { 
        label: 'Select All', 
        accelerator: 'CmdOrCtrl+A', 
        selector: 'selectAll:' 
      }] 
    }];
    var osxMenu = electron.Menu.buildFromTemplate(template);
    electron.Menu.setApplicationMenu(osxMenu);
  }
  else
  {
    // mainWindow.setMenu(null);
  }
  mainWindow.on('closed', function () 
  {
    ApiInterface.killThreads();
    mainWindow = null;
  })
  mainWindow.maximize();

  mainWindow.webContents.on('will-navigate', ev => {
    ev.preventDefault()
  })
  ApiInterface.setMainWindow(mainWindow,__dirname,{platform:process.platform,version:app.getVersion(),path:app.getPath("appData")});
  ApiInterface.setAppVariable(app);
  if(process.argv[2] == '--dev')
  {
    ApiInterface.setDevMode(true);
  }
  

  mainWindow.webContents.session.clearStorageData();
}

app.on('ready', createWindow)

app.on('window-all-closed', function () {
  ApiInterface.killThreads();
    app.quit();
})

app.on('activate', function () {
  if (mainWindow === null) {
    createWindow()
  }
})
